:- module(artofprolog, [subst/4,ground_subst/4,flatten/2,freeze_term/2,melt_new/2]).

% 9.3 A program for substituting in a term
subst(X, Y, Old, New) :-
	freeze_term(Old, Old1), ground_subst(X, Y, Old1, Old2),
	melt_new(Old2, New).

ground_subst(Old, New, Old, New).
ground_subst(Old, _, Term, Term) :-
	constant(Term), Term \= Old.
ground_subst(Old, New, Term, Term1) :-
	compound(Term),
	functor(Term, F, N),
	functor(Term1, F, N),
	ground_subst(N, Old, New, Term, Term1).

ground_subst(N, Old, New, Term, Term1) :-
	N > 0,
	arg(N, Term, Arg),
	ground_subst(Old, New, Arg, Arg1),
	arg(N, Term1, Arg1),
	N1 is N-1,
	ground_subst(N1, Old, New, Term, Term1).
ground_subst(0, _, _, _, _).


% 15.2 Flattening a list of lists using difference-lists
flatten(Xs, Ys) :- flatten_dl(Xs, Ys, []).

flatten_dl([X|Xs], Ys, Zs) :-
	flatten_dl(X, Ys, Ys1), flatten_dl(Xs, Ys1, Zs).
flatten_dl(X, [X|Xs], Xs) :-
	nonlist(X), X \= [].
flatten_dl([], Xs, Xs).

nonlist('.').
nonlist(X) :- \+functor(X, '.', _).

% 15.10 Freezing and melting a term
freeze_term(A, B) :-
	copy_term(A, B), numbervars(B, 0, _).

melt_new(A, B) :-
	melt(A, B, _), !.

melt('$VAR'(N), X, Dictionary) :-
	lookup(N, Dictionary, X).
melt(X, X, _) :-
	constant(X).
melt(X, Y, Dictionary) :-
	compound(X),
	functor(X, F, N),
	functor(Y, F, N),
	melt(N, X, Y, Dictionary).

melt(N, X, Y, Dictionary) :-
	N > 0,
	arg(N, X, ArgX),
	melt(ArgX, ArgY, Dictionary),
	arg(N, Y, ArgY),
	N1 is N-1,
	melt(N1, X, Y, Dictionary).
melt(0, _, _, _).

lookup(Key, dict(Key,X,_,_), Value) :-
	!, X = Value.
lookup(Key, dict(Key1,_,Left,_), Value) :-
	Key < Key1, lookup(Key, Left, Value).
lookup(Key, dict(Key1,_,_,Right), Value) :-
	Key > Key1, lookup(Key, Right, Value).

% ----------------------------------------------------------------------
% Miscellany

constant(X) :- atomic(X).




