:- use_module(library(tcltk)).
:- use_module(artofprolog).

:- ensure_loaded([lifecycle,generator,refiner]).

:- dynamic lifecycle/2, buttons/1, ontologyName/1, ontology/1.

go :-
	clearKnowledgeBase,
	tk_new([name('F-Life')], Tcl),
	tcl_eval(Tcl, 'source interface.tcl', _),
	tk_main_loop,
	tcl_delete(Tcl).
%	halt.

clearKnowledgeBase :-
	retractall(ontologyName(_)),
	retractall(ontology(_)),
	retractall(lifecycle(_,_)),
	retractall(buttons(_)),
	assert(buttons(0)).	% initialised only on start of F-Life session

%%% ----------------------------------------------------------------------
%%% Main menu callbacks from Tcl/Tk interface.tcl

%%% File->New: new lifecycle and tcl-command for GUI update
newLifeCycle(TextWidget, Name, TclCommand) :-
	retractall(lifecycle(_,_)),
	getEmptyLifeCycle(Name, LifeCycle),
	assert(LifeCycle),
	getLifeCycleSpec(LifeCycle, Spec),
	updateDisplay(TextWidget, Spec, TclCommand).

%%% File->Save As...: save lifecycle edit area on file
saveFileAs(File) :-
	lifecycle(Ontology, Spec),
	open(File, write, S),
	write(S, ':- multifile lifecycle/2.\n'),
	write(S, ':- dynamic lifecycle/2.\n'),
	portray_clause(S, lifecycle(Ontology,Spec)),
	close(S).

%%% File->Open: open lifecycle edit area from file
openFile(TextWidget, File, TclCommand) :-
	retractall(lifecycle(_,_)),
	consult(File),
	lifecycle(_, Spec),
	updateDisplay(TextWidget, Spec, TclCommand).

%%% File->Publish: publish the life cycle
publishLifeCycle(File) :-
	lifecycle(Onto, Spec),
	getNameArity(Spec, Name, Arity),
	skolemize(Spec, SkSpec),
	open(File, write, P),
	write(P, ':- op(800, xfx, [=>]).\n\n'),
	write(P, ':- dynamic lifecycle/2, '),
	write(P, Name), write(P, '/'), write(P, Arity), write(P, '.\n\n'),
	portray_clause(P, lifecycle(Onto, SkSpec)),
	close(P).

%%% Ontology->Load: loads life-cycle ontology from file
loadOntology(File) :-
	retractall(ontologyName(_)),
	retractall(ontology(_)),
	assert(ontologyName(File)),
	open(File, read, O),
	repeat,
	  read(O, Clause),
	  (Clause=(_:-_) -> assertz(ontology(Clause))
	                  ; assertz(ontology((Clause:-true)))),
	  at_end_of_stream(O),
	!,
	close(O).

%%% Ontology->Unload: unloads life-cycle ontology from knowledge base
unloadOntology(O) :-
	retract(ontologyName(O)),
	retractall(ontology(_)).

%%% ----------------------------------------------------------------------
%%% Callback from Tcl/Tk listboxes for life-cycle specification refinement

refine(Head, FrozenL) :-
	melt_new(Head, ThawedHead),
	setof(writeq((ThawedHead:-Body)), ontology((ThawedHead:-Body)), L),
	freeze_term(L,FrozenL).
