%%% BASIC LIFE-CYCLE ONTOLOGY

%%% ------------------------------------------------------------------
%%% knowledge transformation

knowledge_transformation(C1,C2) :-
	specification_strengthening(C1,C2),
	knowledge_component(C1),
	knowledge_component(C2).
knowledge_transformation(C1,C2) :-
	specification_weakening(C1,C2),
	knowledge_component(C1),
	knowledge_component(C2).
knowledge_transformation(C1,C2) :-
	language_strengthening(F,C1,C2),
	language_morphism(F),
	knowledge_component(C1),
	knowledge_component(C2).
knowledge_transformation(C1,C2) :-
	language_weakening(F,C1,C2),
	language_morphism(F),
	knowledge_component(C1),
	knowledge_component(C2).
knowledge_transformation(C1,C2) :-
	knowledge_transformation(C1,C3),
	knowledge_transformation(C3,C2).

%%% ------------------------------------------------------------------
%%% life-cycle calculus

specification_strengthening(C1,C2) :-
	subsumes(C2,C1).

specification_weakening(C1,C2) :-
	subsumes(C1,C2).

language_strengthening(morph(L1,L2,P),comp(L1,S1),comp(L2,S2)) :-
	direct_image(morph(L1,L2,P),S1,S2).

language_weakening(morph(L2,L1,P),comp(L1,S1),comp(L2,S2)) :-
	inverse_image(morph(L2,L1,P),S1,S2).

%%% ------------------------------------------------------------------
%%% knowledge component

knowledge_component(comp(L,S)) :-
	language(L),
	specification(S).

%%% ------------------------------------------------------------------
%%% language

language(lang(I,O)) :-
	logical_system(I),
	ontology(O).

ontology(O) :-
	knowledge_component(O).

%%% ------------------------------------------------------------------
%%% language morphism

language_morphism(morph(L1,L2,P)) :-
	language(L1),
	language(L2),
	properties(P).

properties(P) :- type_injective(P).
properties(P) :- type_surjective(P).
properties(P) :- type_bijective(P).
properties(P) :- token_injective(P).
properties(P) :- token_surjective(P).
properties(P) :- token_bijective(P).

type_injective(P) :- type_inclusive(P).

type_bijective(P) :- type_injective(P), type_surjective(P).

token_injective(P) :- token_inclusive(P).

token_bijective(P) :- token_injective(P), token_surjective(P).

%%% ------------------------------------------------------------------
%%% subsumption

%%% of knowledge components:

subsumes(C,C).
subsumes(C1,C3) :- subsumes(C1,C2), subsumes(C2,C3).

subsumes(comp(L,S1),comp(L,S2)) :- % components of the same lagnuage
	subsumes_spec(S1,S2).
subsumes(comp(L1,S1),comp(L2,S2)) :- % components of different languages
	direct_image(morph(L2,L1,_),S2,S3),
	subsumes_spec(S1,S3).

%%% of specifications:

subsumes_spec(S,S).
subsumes_spec(S1,S3) :- subsumes_spec(S1,S2), subsumes_spec(S2,S3).








