#!/usr/local/bin/wish

set fonta -adobe-helvetica-bold-r-normal-*-12-*-*-*-p-*-iso8859-1
set fontb -adobe-helvetica-medium-r-normal-*-12-*-*-*-p-*-iso8859-1
set fontc -adobe-courier-medium-r-normal-*-12-*-*-*-m-*-iso8859-1

# ====================================================================
# SET UP INTERFACE

# MAIN MENU
frame .mm

menubutton .mm.file -menu .mm.file.m -text "File" -font $fonta
menu .mm.file.m -font $fonta
.mm.file.m add command -label "New" -command { new }
.mm.file.m add command -label "Open" -command { openn }
.mm.file.m add command -label "Close"
.mm.file.m add separator
.mm.file.m add command -label "Save" 
.mm.file.m add command -label "Save As..." -command { saveas }
.mm.file.m add command -label "Publish" -command { publish }
.mm.file.m add separator
.mm.file.m add command -label "Exit" -command { destroy . }

menubutton .mm.edit -menu .mm.edit.m -text "Edit" -font $fonta
menu .mm.edit.m -font $fonta
.mm.edit.m add command -label "Undo"
.mm.edit.m add command -label "Redo"

menubutton .mm.onto -menu .mm.onto.m -text "Ontology" -font $fonta
menu .mm.onto.m -font $fonta
.mm.onto.m add command -label "Load" -command { load }
.mm.onto.m add command -label "Unload" -command { unload }

pack .mm.file .mm.edit .mm.onto -in .mm -side left -anchor nw

# EDITING AREA
frame .ed
text .ed.txt -height 40 -width 80 -yscrollcommand ".ed.sy set" -font $fontc
scrollbar .ed.sy -command ".ed.txt yview"
pack .ed.txt .ed.sy -in .ed -side left -anchor nw -fill y

# pack main menu and editing area into the main window
pack .mm .ed -anchor nw

# global variables
set textwidget .ed.txt


# ====================================================================
# PROCEDURES

# main menu option: File->New
proc new { } {
    toplevel .new
    pack [input .new "Life-Cycle Name:" lifecycle]\
	 [appcan .new { new_lifecycle $lifecycle }]
}

proc new_lifecycle { lifecycle } {
    global fonta textwidget

    prolog "newLifeCycle('$textwidget','$lifecycle',TclCommand)"
    clear_text $textwidget
    eval $prolog_variables(TclCommand)
    destroy .new
}

# main menu option: File->Open
proc openn { } {
    toplevel .open
    pack [input .open "File:" file]\
	 [appcan .open { open_file $file }]
}

proc open_file { file } {
    global fonta textwidget

    prolog "openFile('$textwidget','${file}.pl',TclCommand)"
    clear_text $textwidget
    eval $prolog_variables(TclCommand)
    destroy .open
}

# main menu option: File->Save As...
proc saveas { } {
    toplevel .saveas
    pack [input .saveas "File:" file]\
	 [appcan .saveas { save_file_as $file }]
}

proc save_file_as { file } {
    prolog "saveFileAs('${file}.pl')"
    destroy .saveas
}

# main menu option: File->Publish...
proc publish { } {
    toplevel .publish
    pack [input .publish "File:" pfile]\
	 [appcan .publish { publish_lifecycle $pfile }]
}

proc publish_lifecycle { pfile } {
    prolog "publishLifeCycle('${pfile}.pl')"
    note "Life cycle published!"
    destroy .publish
}

# main menu option: Ontology->Load
proc load { } {
    toplevel .load
    pack [input .load "File:" ontology]\
	 [appcan .load { load_ontology $ontology }]
}

proc load_ontology { ontology } {
    global fonta

    prolog "loadOntology('${ontology}.pl')"
    destroy .load
    label .ont -text "Life-cycle ontology: $ontology" -font $fonta
    pack .ont -anchor nw
}

proc unload { } {
    prolog "unloadOntology(O)"
    destroy .ont
}

# ----------------------------------------------------------------------
# callbacks from the main lifecycle production buttons

proc refine { n conjunct } {
    global fonta textwidget

    toplevel .refine
    listbox .refine.lb -width 80 -xscrollcommand ".refine.sx set" -font $fonta
    scrollbar .refine.sx -orient horizontal -command ".refine.lb xview"
    if [prolog "refine($conjunct,L)"] {
	set options $prolog_variables(L)
	foreach opt $options {
	    .refine.lb insert end $opt
	}
    } else {
	set options {}
    }
    .refine.lb insert end "do not refine"
    pack .refine.lb .refine.sx -fill x
    pack [appcan .refine "replace .refine $n {$options}"]
}

# calls Prolog's replace/4 goal and editarea update
proc replace { path n options } {
    global fonta textwidget

    set opt [lindex $options [$path.lb curselection]]
    if {$opt == ""} {
	set opt do_not_refine
    }
    prolog "replace('$textwidget',$n,($opt),TclCommand)"
    clear_text $textwidget
    eval $prolog_variables(TclCommand)
    destroy $path
}

# ======================================================================
# miscellany

# clear editarea
proc clear_text { text } {
    $text delete 1.0 end
}

# pair of label/entry widgets for input
proc input { path text input } {
    global fonta fontb $input

    frame $path.$input
    label $path.$input.l -text $text -font $fonta
    entry $path.$input.e -width 25 -textvariable $input -font $fontb
    pack $path.$input.l $path.$input.e -in $path.$input -side left

    return $path.$input
}

# pair of apply/cancel buttons    
proc appcan { path command } {    
    global fonta

    frame $path.ac
    button $path.ac.apply -text "Apply" -command $command -font $fonta
    button $path.ac.cancel -text "Cancel" -command "destroy $path" -font $fonta
    pack $path.ac.apply $path.ac.cancel -in $path.ac -side left

    return $path.ac
}

# window with message
proc note { text } {
    global fonta

    toplevel .note
    label .note.l -text $text -font $fonta
    button .note.b -text "OK" -command {destroy .note} -font $fonta
    pack .note.l .note.b
}







