:-use_module(misc).

updateDisplay(TextWidget, Spec, TclCommand) :-
	generateImplication(TextWidget, Spec, 0, TclCommand, []).

%%% ----------------------------------------------------------------------
%%% generateImplication(+TextWidget, +Spec, -TclCommand) :-
%%%         TclCommand contains the tcl-commands that display the
%%%         life-cyle specification Spec onto the text widget TextWidget

generateImplication(TextWidget, Antecedent=>Succedent, 0) -->
	generateAntecedent(TextWidget, Antecedent, 0),
	generateSuccedent(TextWidget, Succedent, 1).

generateAntecedent(TextWidget, Antecedent, _) -->
	[';',TextWidget,insert,end,dq(write(Antecedent)),
	 ';',TextWidget,insert,end,dq(write(' => '))].
	
generateSuccedent(TextWidget, exists(Vars,Conj), N) -->
	generateExistential(TextWidget, Vars, N),
	generateConjunction(TextWidget, Conj, N).

generateExistential(TextWidget, [], _) -->
	[';',TextWidget,insert,end,dq('\n')].
generateExistential(TextWidget, Vars, _) -->
	{Vars \= []},
	[';',TextWidget,insert,end,br([' exists ',write(Vars),'\n'])].
	
generateConjunction(_, [], _) --> !.
generateConjunction(TextWidget, [C], N) --> !,
	generateConjunct(TextWidget, C, N, '\n').
generateConjunction(TextWidget, [C|Cs], N) -->
	generateConjunct(TextWidget, C, N, ' and\n'),
	{N1 is N + 1},
	generateConjunction(TextWidget, Cs, N1).

generateConjunct(TextWidget, 'NT'(C), N, Sep) --> % non-terminals are displayed as buttons
	{nameButton(TextWidget,ButtonWidget)},
	[';',button,ButtonWidget,
	 min(text),dq([write(C)]),
	 min(font),write('$fonta'),
	 min(command),br([refine,write(N),br(write_canonical(C))]),
         ';',TextWidget,insert,end,dq(write('\t')),
	 ';',TextWidget,window,create,end,min(window),ButtonWidget,
	 ';',TextWidget,insert,end,dq(write(Sep))].
generateConjunct(TextWidget, C, _, Sep) --> % terminals are display as they are
	[';',TextWidget,insert,end,dq(write('\t')),
	 ';',TextWidget,insert,end,dq(write(C)),
	 ';',TextWidget,insert,end,dq(write(Sep))].

%%% ------------------------------------------------------------------
	
nameButton(TextWidget, ButtonWidget) :-
	retract(buttons(N)),
	N1 is N + 1,
	assert(buttons(N1)),
	atom_number(Num, N1),
	atom_concat('.b', Num, Button),
	atom_concat(TextWidget, Button, ButtonWidget).





