knowledge_transformation(K1,K2) :-
        input_knowledge(K1),
	output_knowledge(K2),
	knowledge_transformation_type(K1,K2).

%%% ------------------------------------------------------------------


%input_knowledge([]).
%input_knowledge([K|Ks]) :- knowledge(K), input_knowledge(Ks).
input_knowledge(K) :- knowledge(K).

output_knowledge(K) :- knowledge(K).

knowledge(k(A,R)) :- abstraction_level(A), representation_level(R).

abstraction_level(A) :- shared_domain_abstraction(A).
abstraction_level(A) :- instance_abstraction_level(A).
abstraction_level(A) :- exclusive_domain_abstraction_level(A).
abstraction_level(A) :- universal_abstraction_level(A).

representation_level(R) :- informal(R).
representation_level(R) :- semi_formal(R).
representation_level(R) :- formal(R).

informal(R) :- free_text(R).
informal(R) :- video(R).

semi_formal(R) :- annotated_text(R).

formal(R) :- structured_represenation(R).
formal(R) :- logical_represenation(R).
formal(R) :- associationist_represenation(R).
formal(R) :- procedural_represenation(R).

structured_represenation(R) :- script_based(R).
structured_represenation(R) :- mark_up(R).
structured_represenation(R) :- attribute_value_pairs(R).
structured_represenation(R) :- tabular_data(R).
structured_represenation(R) :- rule_based(R).
structured_represenation(R) :- frame_based(R).

mark_up(R) :- rdf(R).
mark_up(R) :- xml(R).

attribute_value_pairs(R) :- numeric_value_pairs(R).
attribute_value_pairs(R) :- symbolic_value_pairs(R).
attribute_value_pairs(R) :- mixed_symbolic_numeric_value_pairs(R).

logical_represenation(R) :- higher_order_logic(R).
logical_represenation(R) :- predicate_logic(R).
logical_represenation(R) :- first_order_logic(R).
logical_represenation(R) :- extended_fol(R).

% possible limk with marco's lcc?
% logical_represenation(comp(L,S)) :-
%	language(L),
%	specification(S).
%
% language(lang(I,O)) :-
%	logical_system(I),
%	ontology(O).
%
% logical_system(I) :- higher_order_logic(I).
% logical_system(I) :- predicate_logic(I).
% logical_system(I) :- firts_order_logic(I).
% logical_system(I) :- extended_fol(I).

extended_fol(R) :- kif(R).
extended_fol(R) :- ontolingua(R).

associationist_representation(R) :- bayesian_network(R).
associationist_representation(R) :- semantic_network(R).
associationist_representation(R) :- connectionist_network(R).

procedural_representation(R) :- rule_based(R).
procedural_representation(R) :- production_system(R).
procedural_representation(R) :- connectionist_network(R).

%%% ------------------------------------------------------------------
knowledge_transformation_type(K1,K2) :- transformations_for_knowledge_structuring(K1,K2).
knowledge_transformation_type(K1,K2) :- transformations_for_knowledge_sharing(K1,K2).
knowledge_transformation_type(K1,K2) :- transformations_for_knowledge_reuse(K1,K2).

transformations_for_knowledge_structuring(K1,K2) :- knowledge_deletion(K1,K2).
transformations_for_knowledge_structuring(K1,K2) :- knowledge_creation(K1,K2).
transformations_for_knowledge_structuring(K1,K2) :- knowledge_content_refinement(K1,K2).

knowledge_content_refinement(K1,K2):- knowledge_content_refinment(K1,K2).
knowledge_content_refinement(K1,K2):- knowledge_content_addition(K1,K2).
knowledge_content_refinement(K1,K2):- knowledge_content_alteration(K1,K2).

transformations_for_knowledge_sharing(K1,K2) :- semantic_alteration(K1,K2).
transformations_for_knowledge_sharing(K1,K2) :- semantic_reconciliation(K1,K2).
transformations_for_knowledge_sharing(K1,K2) :- syntactic_reconciliation(K1,K2).

semantic_alteration(K1,K2) :- segmentation(K1,K2).
semantic_alteration(K1,K2) :- generalisation_synthesis(K1,K2).
semantic_alteration(K1,K2) :- view_generation(K1,K2).
semantic_alteration(K1,K2) :- abstraction(K1,K2).

semantic_reconciliation(K1,K2) :- semantic_merging(K1,K2).
semantic_reconciliation(K1,K2) :- semantic_translation(K1,K2).

semantic_translation(K1,K2) :- nonlossy_semantic_translation(K1,K2).
semantic_translation(K1,K2) :- lossy_semantic_translation(K1,K2).

syntactic_reconciliation(K1,K2) :- syntactic_translation(K1,K2).

syntactic_translation(K1,K2) :- lossy_syntactic_translation(K1,K2).
syntactic_translation(K1,K2) :- nonlossy_syntactic_translation(K1,K2).

transformations_for_knowledge_reuse(K1,K2) :- semantic_reformulation(K1,K2).
transformations_for_knowledge_reuse(K1,K2) :- semantic_construction(K1,K2).

semantic_construction(K1,K2) :- knowledge_integration(K1,K2).
semantic_construction(K1,K2) :- knowledge_customisation(K1,K2).
semantic_construction(K1,K2) :- knowledge_composition(K1,K2).

knowledge_customisation(K1,K2) :- custom_extension(K1,K2).
knowledge_customisation(K1,K2) :- custom_modification(K1,K2).
knowledge_customisation(K1,K2) :- custom_contraction(K1,K2).

































