:- op(800, xfx, [=>]).

:- use_module(library(lists)).

:- dynamic loadedFiles/1, pclause/1.
:- assert(loadedFiles([])).

/*
    prove(Ontology, LifeCycle, Property) :-
	    Property holds with respect to LifeCycle and Ontology
	    
    Ontology is a files containing ontological definitions
    LifeCycle is a file containing a life-cycle specification
    Property is the property to be proved

    Example:
    prove('euzenat.pl', 'boo.pl', boo(A,B)=>[interpretation_preserving_transformation(A,B)]).
*/

prove(Ontology, LifeCycle, Property) :-
	loadFile(Ontology),
	loadLifeCycle(LifeCycle),
	proveProperty(Property).

%%% ------------------------------------------------------------------
%%% load files into knowledge base

loadFiles([]).
loadFiles([F|Fs]) :- loadFile(F), loadFiles(Fs).

loadFile(File) :-
	loadedFiles(Loaded),
	\+ member(File, Loaded), !,
	open(File, read, O),
	repeat,
	  read(O, Clause),
	  process(Clause),
	  at_end_of_stream(O),
	!,
	close(O),
	retract(loadedFiles(Files)),
	assert(loadedFiles([File|Files])).
loadFile(_).

process((:-load_files(Os))) :- !, loadFiles(Os).
process((Head:-Body)) :- !, assertz(pclause((Head:-Body))).
process(Clause) :- assertz(pclause((Clause:-true))).

%%% ------------------------------------------------------------------
%%% load a lifecycle and add clauses to knowledge base

loadLifeCycle(LifeCycle) :-
	consult(LifeCycle),
	lifecycle2clauses.

lifecycle2clauses :-
	retract(lifecycle(Onto,Ant=>Suc)), !,
	loadFile(Onto),
	addClauses(Suc, Ant),
	lifecycle2clauses.
lifecycle2clauses.

addClauses([], _).
addClauses([S|Suc], Ant) :-
	assertz(pclause((S:-Ant))),
	addClauses(Suc, Ant).

%%% ------------------------------------------------------------------
%%% prove a property with vanilla meta-interpreter

proveProperty(Ant=>Suc) :-
	numbervars(Ant, 0, _),	% freezes the term Ant
	assertz(pclause((Ant:-true))),
	proveSuccedent(Suc).

proveSuccedent([]).
proveSuccedent([S|Suc]) :- solve(S), proveSuccedent(Suc).

solve(true).
solve((A,B)) :- solve(A), solve(B).
solve(A) :- pclause((A:-B)), solve(B).
			 
