:- op(800, xfx, [=>]).

:- use_module(library(lists)).
:- use_module(library(terms)).
:- use_module(artofprolog).

%%% ------------------------------------------------------------------
%%% replace(+TextWidget, +N, +Option, -TclCommand) :-
%%%     replaces the N-th literal of the succedent of the current
%%%     life-cycle specification with the list of literals stated in
%%%     Options and constructs tcl-command TclCommand for updating
%%%     TextWidget on the interface

replace(TextWidget, N, Option, TclCommand) :-
	retract(lifecycle(Ontology,Spec)),
	melt_new(Spec, Thawed),
	rewriteSpec(Thawed, N, Option, NewSpec),
	freeze_term(NewSpec, Frozen),
	assert(lifecycle(Ontology,Frozen)),
	updateDisplay(TextWidget, Frozen, TclCommand).

%%% ------------------------------------------------------------------

rewriteSpec(Ant=>exists(_,Suc), N, Option, Ant=>exists(NewVars,NewSuc)) :-
	rewriteSuc(Suc, N, Option, NewSuc),
	term_variables(Ant, VA), 
	term_variables(NewSuc, VS),
	delete_list(VS, VA, NewVars). % isolates variables not in antecedent 

rewriteSuc(Suc, N, do_not_refine, NewSuc) :-
	split(Suc, N, SBefore, 'NT'(C), SAfter),
	append(SBefore, [C], SL),
	append(SL, SAfter, NewSuc).
rewriteSuc(Suc, N, Option, NewSuc) :-
%	melt_new(Option, (Head:-Body)),
	Option = (Head:-Body),
	split(Suc, N, SBefore, 'NT'(Head), SAfter),
	body2ntlist(Body, L),
	append(SBefore, L, SL),
	append(SL, SAfter, NewSuc).

%%% ------------------------------------------------------------------

delete_list(L, [], L).
delete_list(L1, [X|L], L3):-
	delete(L1, X, L2),
	delete_list(L2, L, L3).

split([X|L], 1, [], X, L).
split([X|L], N, [X|L1], Y, L2) :-
	N > 1,
	N1 is N-1,
	split(L, N1, L1, Y, L2).

body2ntlist(true, []):- !.
body2ntlist(B, ['NT'(B)]) :- B \= (_,_).
body2ntlist((B,Bs), ['NT'(B)|LBs]) :- body2ntlist(Bs, LBs).










